#!/usr/bin/env python3
"""
Model classes for job control
"""
from marshmallow import Schema, fields, post_load


class DependencySchema(Schema):
    modified = fields.String()
    job_id = fields.Integer()


class JobSchema(Schema):
    job_id = fields.Integer()
    name = fields.String()
    username = fields.String()
    work_dir = fields.String()
    state = fields.String()
    partition = fields.String()
    account = fields.String()
    reason = fields.String()
    reservation = fields.String()
    time_left = fields.String()
    time_limit = fields.String()
    time_used = fields.String()
    command = fields.String()
    comment = fields.String()
    dependency = fields.List(fields.Nested(DependencySchema))
    end_time = fields.String()
    fields.exit_code = fields.Integer()
    fields.gres = fields.List(fields.String())

    @post_load
    def create_job(self, data, **kwargs):
        return Job(**data)


class Job(object):
    def __init__(self,
                 job_id=None,
                 name=None,
                 username=None,
                 work_dir=None,
                 state=None,
                 partition=None,
                 account=None,
                 reason=None,
                 reservation=None,
                 time_left=None,
                 time_limit=None,
                 time_used=None,
                 command=None,
                 comment=None,
                 dependency=None,
                 endtime=None,
                 exit_code=None,
                 gres=None):
        self.job_id = job_id
        self.name = name
        self.username = username
        self.work_dir = work_dir
        self.state = state
        self.partition = partition
        self.account = account
        self.reason = reason
        self.reservation = reservation
        self.time_left = time_left
        self.time_limit = time_limit
        self.time_used = time_used
        self.command = command
        self.comment = comment
        self.dependency = dependency
        self.end_time = endtime
        self.exit_code = exit_code
        self.gres = gres

    def __repr__(self):
        return f"Job(job_id={self.job_id}, username={self.username})"
