#!/bin/bash
# Load modules
echo Loading modules...
module purge

export SOFTWARE_STAGE=2020
export OTHERSTAGES=/p/software/jurecadc/otherstages
export SOFTWARE_DIR=/p/project/cjinm16/schiffer1/software
export SOFTWARE_STAGES_DIR=${SOFTWARE_DIR}/stages/
export SOFTWARE_STAGE_DIR=${SOFTWARE_STAGES_DIR}/${SOFTWARE_STAGE}
export SOFTWARE_STAGE_MODULES=${SOFTWARE_STAGE_DIR}/modules/
export ATLAS_MODULE=atlas/OpenMPI_PyTorch

module use $OTHERSTAGES &> /dev/null
module use ${SOFTWARE_STAGE_MODULES} &> /dev/null

module load ${ATLAS_MODULE}
echo Modules loaded

# Environment variables
export BRAIN=__BRAIN__
export LABELS=__LABELS__
export SCRATCH_USER=/p/scratch/cjinm16/schiffer1/

# Set annotation directory (assume we are in the experiment folder)
export ATLAS_ANNOTATION_DIR=`pwd`/annotations/
export ATLAS_ANNOTATION_FILE="B${BRAIN}_{slice:04d}_annotations.hdf5"

# Some more variables
INPUT_SPACING=1
OUTPUT_SPACING=16
PREFIX="mask"

# Get path to TIFF images to determine shape
IMAGE_PATTERN=`python -c "from atlas import constants; print(constants.original_path.format(brain=int('${BRAIN}'), slice=1234))"`
echo "Image pattern: ${IMAGE_PATTERN}"

ANNOTATION_PATTERN=`python -c "from atlas import constants; print(constants.annotation_path.format(brain=int('${BRAIN}'), slice=1234))"`
echo "Annotation pattern: ${ANNOTATION_PATTERN}"

# Setup the annotation data
atlas img ann2hdf5 ${ATLAS_ANNOTATION_DIR}/*.json \
                   ${IMAGE_PATTERN} \
                   ${ANNOTATION_PATTERN} \
                   --input-spacing ${INPUT_SPACING} \
                   --output-spacing ${OUTPUT_SPACING} \
                   --prefix ${PREFIX} \
                   --overwrite \
                   --labels ${LABELS} \
                   --chunk-size 64 \
                   --compression gzip

# Setup the experiment data
atlas exp run new `pwd`

# Train network
srun --cpus-per-task 4 atlas dl train `pwd` --verbose --distribute
